@@ -19,8 +19,10 @@ from pandas.api.types import (
 )
 from pandas.core.arrays import ExtensionArray
 from pandas.core.dtypes.dtypes import ExtensionDtype, register_extension_dtype
+from pandas.util._decorators import doc
 
 from fletcher._algorithms import (
+    extract_isnull_bytemap,
     kurt_op,
     max_op,
     median_op,
@@ -30,6 +32,7 @@ from fletcher._algorithms import (
     skew_op,
     std_op,
     sum_op,
+    take_on_pyarrow_list,
     var_op,
 )
 from fletcher.algorithms.bool import all_op, all_true, any_op, or_na, or_vectorised
@@ -136,6 +139,8 @@ def _is_numeric(arrow_dtype: pa.DataType) -> bool:
 class FletcherBaseDtype(ExtensionDtype):
     """Dtype base for a pandas ExtensionArray backed by an Apache Arrow structure."""
 
+    na_value = pd.NA
+
     def __init__(self, arrow_dtype: pa.DataType):
         self.arrow_dtype = arrow_dtype
 
@@ -513,6 +518,13 @@ class FletcherBaseArray(StringSupportingExtensionArray):
             )
         )
 
+    def _as_pandas_scalar(self, arrow_scalar: pa.Scalar):
+        scalar = arrow_scalar.as_py()
+        if scalar is None:
+            return self._dtype.na_value
+        else:
+            return scalar
+
     def __array_ufunc__(self, ufunc, method: str, *inputs, **kwargs):
         """Apply a NumPy ufunc on the ExtensionArray."""
         if method != "__call__":
@@ -746,6 +758,13 @@ class FletcherBaseArray(StringSupportingExtensionArray):
             indices_array = pa.array(indices, mask=mask)
         elif is_array_like(indices) and len(indices) == 0:
             indices_array = pa.array([], type=pa.int64())
+        elif (
+            self.dtype.is_list
+            and self.data.flatten().null_count == 0
+            and self.data.null_count == 0
+            and self.data.flatten().dtype._is_numeric
+        ):
+            return take_on_pyarrow_list(self.data, indices)
         else:
             raise NotImplementedError(f"take is not implemented for {type(indices)}")
         return type(self)(array.take(indices_array))
@@ -854,7 +873,11 @@ class FletcherBaseArray(StringSupportingExtensionArray):
 
         This should return a 1-D array the same length as 'self'.
         """
-        return np.array(self.data.is_null())
+        if hasattr(self.data, "is_null"):
+            return np.array(self.data.is_null())
+        else:
+            # Remove once drop support for pyarrow<1
+            return extract_isnull_bytemap(self.data)
 
 
 class FletcherContinuousArray(FletcherBaseArray):
@@ -864,7 +887,7 @@ class FletcherContinuousArray(FletcherBaseArray):
         # Copy is not used at the moment. It's only affect will be when we
         # allow array to be a FletcherContinuousArray
         if is_array_like(array) or isinstance(array, list):
-            self.data = pa.array(array, type=dtype)
+            self.data = pa.array(array, type=dtype, from_pandas=True)
         elif isinstance(array, pa.Array):
             # TODO: Assert dtype
             self.data = array
@@ -941,6 +964,8 @@ class FletcherContinuousArray(FletcherBaseArray):
             key_array = np.asanyarray(key)
 
         if pd.api.types.is_scalar(value):
+            if value is pd.NA:
+                value = None
             value = np.broadcast_to(value, len(key_array))
         else:
             value = np.asarray(value)
@@ -1041,7 +1066,7 @@ class FletcherContinuousArray(FletcherBaseArray):
         if isinstance(value, pa.Array):
             return type(self)(value)
         else:
-            return value.as_py()
+            return self._as_pandas_scalar(value)
 
     def copy(self):
         # type: () -> ExtensionArray
@@ -1066,35 +1091,8 @@ class FletcherContinuousArray(FletcherBaseArray):
                 size += buf.size
         return size
 
+    @doc(ExtensionArray.factorize)
     def factorize(self, na_sentinel=-1):
-        # type: (int) -> Tuple[np.ndarray, ExtensionArray]
-        """Encode the extension array as an enumerated type.
-
-        Parameters
-        ----------
-        na_sentinel : int, default -1
-            Value to use in the `labels` array to indicate missing values.
-
-        Returns
-        -------
-        labels : ndarray
-            An integer NumPy array that's an indexer into the original
-            ExtensionArray.
-        uniques : ExtensionArray
-            An ExtensionArray containing the unique values of `self`.
-            .. note::
-               uniques will *not* contain an entry for the NA value of
-               the ExtensionArray if there are any missing values present
-               in `self`.
-
-        See Also
-        --------
-        pandas.factorize : Top-level factorize method that dispatches here.
-
-        Notes
-        -----
-        :meth:`pandas.factorize` offers a `sort` keyword as well.
-        """
         if pa.types.is_dictionary(self.data.type):
             indices = self.data.indices.to_pandas()
             return indices.values, type(self)(self.data.dictionary)
@@ -1104,7 +1102,7 @@ class FletcherContinuousArray(FletcherBaseArray):
             indices = encoded.indices.to_pandas()
             if indices.dtype.kind == "f":
                 indices[np.isnan(indices)] = na_sentinel
-                indices = indices.astype(int)
+                indices = indices.astype(np.int64)
             if not is_int64_dtype(indices):
                 indices = indices.astype(np.int64)
             return indices.values, type(self)(encoded.dictionary)
@@ -1126,6 +1124,8 @@ class FletcherContinuousArray(FletcherBaseArray):
         """
         if isinstance(scalars, FletcherContinuousArray):
             return scalars
+        if not ARROW_GE_0_18_0:
+            scalars = [None if x is pd.NA else x for x in scalars]
         if dtype and isinstance(dtype, FletcherContinuousDtype):
             dtype = dtype.arrow_dtype
         return cls(pa.array(scalars, type=dtype, from_pandas=True))
@@ -1263,7 +1263,9 @@ class FletcherChunkedArray(FletcherBaseArray):
         # Copy is not used at the moment. It's only affect will be when we
         # allow array to be a FletcherChunkedArray
         if is_array_like(array) or isinstance(array, list):
-            self.data = pa.chunked_array([pa.array(array, type=dtype)])
+            self.data = pa.chunked_array(
+                [pa.array(array, type=dtype, from_pandas=True)]
+            )
         elif isinstance(array, pa.Array):
             # ARROW-7008: pyarrow.chunked_array([array]) fails on array with all-None buffers
             if len(array) == 0 and all(b is None for b in array.buffers()):
@@ -1472,7 +1474,7 @@ class FletcherChunkedArray(FletcherBaseArray):
         if isinstance(value, pa.ChunkedArray):
             return type(self)(value)
         else:
-            return value.as_py()
+            return self._as_pandas_scalar(value)
 
     def copy(self):
         # type: () -> ExtensionArray
@@ -1501,50 +1503,28 @@ class FletcherChunkedArray(FletcherBaseArray):
                     size += buf.size
         return size
 
+    @doc(ExtensionArray.factorize)
     def factorize(self, na_sentinel=-1):
-        # type: (int) -> Tuple[np.ndarray, ExtensionArray]
-        """Encode the extension array as an enumerated type.
-
-        Parameters
-        ----------
-        na_sentinel : int, default -1
-            Value to use in the `labels` array to indicate missing values.
-
-        Returns
-        -------
-        labels : ndarray
-            An integer NumPy array that's an indexer into the original
-            ExtensionArray.
-        uniques : ExtensionArray
-            An ExtensionArray containing the unique values of `self`.
-            .. note::
-               uniques will *not* contain an entry for the NA value of
-               the ExtensionArray if there are any missing values present
-               in `self`.
-
-        See Also
-        --------
-        pandas.factorize : Top-level factorize method that dispatches here.
-
-        Notes
-        -----
-        :meth:`pandas.factorize` offers a `sort` keyword as well.
-        """
         if pa.types.is_dictionary(self.data.type):
             raise NotImplementedError()
-        elif self.data.num_chunks == 1:
+        else:
             # Dictionaryencode and do the same as above
-            encoded = self.data.chunk(0).dictionary_encode()
-            indices = encoded.indices.to_pandas()
+            encoded = self.data.dictionary_encode()
+            indices = pa.chunked_array(
+                [c.indices for c in encoded.chunks], type=encoded.type.index_type
+            ).to_pandas()
             if indices.dtype.kind == "f":
                 indices[np.isnan(indices)] = na_sentinel
                 indices = indices.astype(int)
             if not is_int64_dtype(indices):
                 indices = indices.astype(np.int64)
-            return indices.values, type(self)(encoded.dictionary)
-        else:
-            np_array = self.data.to_pandas().values
-            return pd.factorize(np_array, na_sentinel=na_sentinel)
+            if encoded.num_chunks == 0:
+                return (
+                    indices.values,
+                    type(self)(pa.array([], type=encoded.type.value_type)),
+                )
+            else:
+                return indices.values, type(self)(encoded.chunk(0).dictionary)
 
     @classmethod
     def _from_sequence(cls, scalars, dtype=None, copy=None):
@@ -1563,6 +1543,8 @@ class FletcherChunkedArray(FletcherBaseArray):
         """
         if isinstance(scalars, FletcherChunkedArray):
             return scalars
+        if not ARROW_GE_0_18_0:
+            scalars = [None if x is pd.NA else x for x in scalars]
         if dtype and isinstance(dtype, FletcherChunkedDtype):
             dtype = dtype.arrow_dtype
         return cls(pa.array(scalars, type=dtype, from_pandas=True))
