@@ -311,6 +311,14 @@ def sort_by_key(request):
     return request.param
 
 
+@pytest.fixture(params=[None, np.nan, pd.NaT, float("nan"), pd.NA], ids=str)
+def nulls_fixture(request):
+    """
+    Fixture for each null type in pandas.
+    """
+    return request.param
+
+
 class TestBaseCasting(BaseCastingTests):
     pass
 
@@ -397,6 +405,17 @@ class TestBaseInterfaceTests(BaseInterfaceTests):
     def test_copy(self, data):
         BaseInterfaceTests.test_array_interface(self, data)
 
+    @xfail_list_equals_not_implemented
+    @pytest.mark.xfail_by_type_filter(
+        [pa.types.is_date], "NaT and null are handled slightly differently"
+    )
+    @pytest.mark.xfail_by_type_filter(
+        [pa.types.is_floating], "NaN and null are handled slightly differently"
+    )
+    def test_contains(self, data, data_missing, nulls_fixture):
+        if hasattr(BaseInterfaceTests, "test_contains"):
+            BaseInterfaceTests.test_contains(self, data, data_missing, nulls_fixture)
+
 
 class TestBaseMethodsTests(BaseMethodsTests):
 
@@ -467,6 +486,7 @@ class TestBaseMethodsTests(BaseMethodsTests):
 
     @pytest.mark.parametrize("ascending", [True, False])
     @xfail_bool_too_few_uniques
+    @xfail_missing_list_dict_encode
     def test_sort_values(self, data_for_sorting, ascending, sort_by_key):
         if PANDAS_GE_1_1_0:
             BaseMethodsTests.test_sort_values(
